//! Main library for AeroLattice.

mod vector3d;

use pyo3::prelude::*;

pub use vector3d::Vector3D;

/// Formats the sum of two numbers as string.
// #[pyfunction]
// fn sum_as_string(a: usize, b: usize) -> PyResult<String> {
//     Ok((a + b).to_string())
// }

/// A Python module implemented in Rust.
#[pymodule]
fn aerolattice(m: &Bound<'_, PyModule>) -> PyResult<()> {
    // m.add_function(wrap_pyfunction!(sum_as_string, m)?)?;
    
    // Add `Vector` class.
    m.add_class::<Vector3D>()?;
    
    Ok(())
}
